package org.dan.lastjcl;

import java.util.Collection;
import java.util.List;

import org.dan.lastjcl.scalcol.BooleansCollection;
import org.dan.lastjcl.scalcol.BytesCollection;
import org.dan.lastjcl.scalcol.CharsCollection;
import org.dan.lastjcl.scalcol.DoublesCollection;
import org.dan.lastjcl.scalcol.FloatsCollection;
import org.dan.lastjcl.scalcol.IntsCollection;
import org.dan.lastjcl.scalcol.LongsCollection;
import org.dan.lastjcl.scalcol.ShortsCollection;
import org.dan.lastjcl.scalist.BooleansList;
import org.dan.lastjcl.scalist.BytesList;
import org.dan.lastjcl.scalist.CharsList;
import org.dan.lastjcl.scalist.DoublesList;
import org.dan.lastjcl.scalist.FloatsList;
import org.dan.lastjcl.scalist.IntsList;
import org.dan.lastjcl.scalist.LongsList;
import org.dan.lastjcl.scalist.ShortsList;
import org.dan.lastjcl.scaliter.BooleansIteratorFactory;
import org.dan.lastjcl.scaliter.BytesIteratorFactory;
import org.dan.lastjcl.scaliter.CharsIteratorFactory;
import org.dan.lastjcl.scaliter.DoublesIteratorFactory;
import org.dan.lastjcl.scaliter.FloatsIteratorFactory;
import org.dan.lastjcl.scaliter.IntsIteratorFactory;
import org.dan.lastjcl.scaliter.LongsIteratorFactory;
import org.dan.lastjcl.scaliter.ShortsIteratorFactory;

/**
 * Transparent way working with scalar arrays. 
 * 
 * There are 3 groups of overloaded methods:
 * <ul>
 * 	
 * </ul>
 * 
 * @author <Daneel S. Yaitskov> rtfm.rtfm.rtfm@gmail.com
 *  2012 year
 * 
 */
public class ScalUtils {
	/**
	 * Wrap a byte array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Byte> wrapIterable(byte[] array) {
		return new BytesIteratorFactory(array);
	}

	/**
	 * Wrap a byte array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Byte> wrapCollection(byte[] array) {
		return new BytesCollection(array);
	}

	/**
	 * Wrap a byte array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Byte> wrapList(byte[] array) {
		return new BytesList(array);
	}

	/**
	 * Wrap a int array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Integer> wrapIterable(int[] array) {
		return new IntsIteratorFactory(array);
	}

	/**
	 * Wrap a int array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Integer> wrapCollection(int[] array) {
		return new IntsCollection(array);
	}

	/**
	 * Wrap a int array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Integer> wrapList(int[] array) {
		return new IntsList(array);
	}

	/**
	 * Wrap a short array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Short> wrapIterable(short[] array) {
		return new ShortsIteratorFactory(array);
	}

	/**
	 * Wrap a short array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Short> wrapCollection(short[] array) {
		return new ShortsCollection(array);
	}

	/**
	 * Wrap a short array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Short> wrapList(short[] array) {
		return new ShortsList(array);
	}

	/**
	 * Wrap a char array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Character> wrapIterable(char[] array) {
		return new CharsIteratorFactory(array);
	}

	/**
	 * Wrap a char array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Character> wrapCollection(char[] array) {
		return new CharsCollection(array);
	}

	/**
	 * Wrap a char array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Character> wrapList(char[] array) {
		return new CharsList(array);
	}

	/**
	 * Wrap a boolean array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Boolean> wrapIterable(boolean[] array) {
		return new BooleansIteratorFactory(array);
	}

	/**
	 * Wrap a boolean array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Boolean> wrapCollection(boolean[] array) {
		return new BooleansCollection(array);
	}

	/**
	 * Wrap a boolean array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Boolean> wrapList(boolean[] array) {
		return new BooleansList(array);
	}

	/**
	 * Wrap a long array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Long> wrapIterable(long[] array) {
		return new LongsIteratorFactory(array);
	}

	/**
	 * Wrap a long array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Long> wrapCollection(long[] array) {
		return new LongsCollection(array);
	}

	/**
	 * Wrap a long array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Long> wrapList(long[] array) {
		return new LongsList(array);
	}

	/**
	 * Wrap a double array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Double> wrapIterable(double[] array) {
		return new DoublesIteratorFactory(array);
	}

	/**
	 * Wrap a double array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Double> wrapCollection(double[] array) {
		return new DoublesCollection(array);
	}

	/**
	 * Wrap a double array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Double> wrapList(double[] array) {
		return new DoublesList(array);
	}

	/**
	 * Wrap a float array into an object with {@link Iterable} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return iterable object wrapper over the {@code array}
	 */
	public static Iterable<Float> wrapIterable(float[] array) {
		return new FloatsIteratorFactory(array);
	}

	/**
	 * Wrap a float array into an object with {@link Collection} interface.
	 * 
	 * Structure modification operations are not supported.
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static Collection<Float> wrapCollection(float[] array) {
		return new FloatsCollection(array);
	}

	/**
	 * Wrap a float array into an object with {@link List} interface. 
	 * 
	 * Structure modification operations are not supported. 
	 * @param array not null
	 * @return wrapper over the {@code array}
	 */
	public static List<Float> wrapList(float[] array) {
		return new FloatsList(array);
	}

	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Byte}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(byte[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}

	/**
	 * Create an array by example of type other array.
	 * @param example any array
	 * @param length of new array
	 * @return new array with type {@code T}
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] newArray(T[] example, int length) {
		return (T[]) java.lang.reflect.Array.newInstance(example.getClass()
				.getComponentType(), length);
	}

	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(byte[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}
	
	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(boolean[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(int[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(char[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(short[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(float[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(long[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}
	
	/**
	 * Behavior is described in {@link Collection#toArray(Object[])}.
	 * @param srcArray like {@code this}
	 * @param dstArray like the argument of {@link Collection#toArray(Object[])}
	 * @return {@code dstArray} or a new array with elements of {@code srcArray}
	 * @see Collection#toArray(Object[]) 
	 */
	@SuppressWarnings("unchecked")
	public static <T> T[] copyItems(double[] srcArray, T[] dstArray) {
		if (dstArray.length < srcArray.length)
			dstArray = newArray(dstArray, srcArray.length);
		int i = 0;
		for (; i < srcArray.length; ++i)
			dstArray[i] = (T) (Object) srcArray[i];
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(byte[] array, byte item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(long[] array, long item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(int[] array, int item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(short[] array, short item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(char[] array, char item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(boolean[] array, boolean item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(double[] array, double item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return first index of element of {@code array} that equals to {@code item}.
	 * @param array not null
	 * @param item searching value
	 * @return first index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#indexOf(Object)
	 */
	public static int indexOf(float[] array, float item) {
		for (int i = 0; i < array.length; ++i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(byte[] array, byte item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(int[] array, int item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}

	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(short[] array, short item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(char[] array, char item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(boolean[] array, boolean item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(float[] array, float item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(double[] array, double item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Return last index of element of {@code array} that equals to {@code item}. 
	 * @param array not null
	 * @param item searching value
	 * @return last index of element of {@code array} that equals to {@code item} 
	 *         or -1 otherwise.
	 * @see List#lastIndexOf(Object)
	 */
	public static int lastIndexOf(long[] array, long item) {
		for (int i = array.length - 1; i >= 0; --i)
			if (array[i] == item)
				return i;
		return -1;
	}
	
	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Integer}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(int[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}

	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Long}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(long[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}

	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Double}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(double[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}

	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Float}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(float[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}

	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Character}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(char[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}

	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Boolean}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(boolean[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}
	
	/**
	 * Copy array of scalar values into an array of object ones. New array has
	 * the same length. A scalar element converted into the object analog -
	 * {@link Short}.
	 * 
	 * @param array not null
	 * @return array of boxed values
	 */
	public static Object[] copyAsObject(short[] array) {
		Object[] result = new Object[array.length];
		for (int i = 0; i < array.length; ++i)
			result[i] = array[i];
		return result;
	}
}