/**
 * 
 */
package org.dan.lastjcl.scalcol;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.dan.lastjcl.IterableUtils;
import org.dan.lastjcl.ScalUtils;
import org.dan.lastjcl.scaliter.CompoundIterable;

/**
 * Concatenate other collections into one without copying them.
 * @author Daneel S. Yaitskov (rtfm.rtfm.rtfm@gmail.com)
 * 2012 year
 *
 */
public class CompoundCollection<T> extends CompoundIterable<T> implements Collection<T> {

	protected int length;
	@SuppressWarnings("unchecked")
	public CompoundCollection(List<Collection<T>> iterables) {
		super((List<Iterable<T>>)(Object)iterables);
		
		length = countSize();
	}
	
	public int countSize() {
		int sum = 0;
		for (Iterable<T> sub : iterables) 
			sum += ((Collection<T>)sub).size();		
		return sum;
	}
	
	/**
	 *
	 */
	public int size() {
		return length;
	}

	/**
	 *
	 */
	public boolean isEmpty() {
		return length == 0;
	}

	/**
	 *
	 */
	public boolean contains(Object o) {
		return IterableUtils.contains(this, o);
	}

	/**
	 *
	 */
	public Object[] toArray() {
		Object[] result = new Object[length];
		Iterator<T> iter = iterator();
		for (int i = 0; i < result.length; ++i)
			result[i] = iter.next();		
		return result;
	}

	/**
	 *
	 */
	@SuppressWarnings("unchecked")
	public <E> E[] toArray(E[] dstArray) {
		if (dstArray.length < this.length)
			dstArray = ScalUtils.newArray(dstArray, this.length);
		int i = 0;
		Iterator<T> iter = iterator();
		for (; i < this.length; ++i)
			dstArray[i] = (E) (Object) iter.next();
		for (; i < dstArray.length; ++i)
			dstArray[i] = null;
		return dstArray;
	}

	
	public void checkNotEmpty() {
		if (iterables.isEmpty())
			throw new IllegalStateException("there is not any sub collection");
	}
	
	/**
	 * delegate to add of last sub collection
	 */
	public boolean add(T e) {		
		if (lastSubCollection().add(e)) {
			length = countSize();
			return true;
		}
		return false;
	}

	/**
	 *
	 */
	public boolean remove(Object o) {		
		if (lastSubCollection().remove(o)) {
			length = countSize();
			return true;
		}
		return false;
	}

	/**
	 *
	 */
	public boolean containsAll(Collection<?> c) {
		return IterableUtils.containsAll(this, c);
	}

	protected Collection<T> lastSubCollection() {
		checkNotEmpty();
		return ((Collection<T>)iterables.get(iterables.size() - 1));
	}
	/**
	 * delegate to addAll of last sub collection.
	 */
	public boolean addAll(Collection<? extends T> c) {		
		if (lastSubCollection().addAll(c)) {
			length = countSize();
			return true;
		}
		return false;
	}

	/**
	 *
	 */
	public boolean removeAll(Collection<?> c) {
		Iterator<T> thisIter = iterator();
		boolean changed = false;		
		while (thisIter.hasNext()) {
			T t = thisIter.next();					
			for (Object o : c) {				
				if (t == null && o == null || t.equals(o)) {
					changed = true;
					thisIter.remove();
					length -= 1;
					break;
				}				
			}
		}		
		return changed;
	}

	/**
	 *
	 */
	public boolean retainAll(Collection<?> c) {
		Iterator<T> thisIter = iterator();
		boolean changed = false;
		next:
		while (thisIter.hasNext()) {
			T t = thisIter.next();			
			for (Object o : c)
				if (t == null && o == null || t.equals(o))
					continue next;	
			changed = true;
			thisIter.remove();
			length -= 1;
		}
		return changed;
	}

	/**
	 * Clears all sub collections
	 */
	public void clear() {
		for (Iterable<T> col : iterables) 
			((Collection<T>)col).clear();
		length = countSize();
	}
}
