/**
 * 
 */
package org.dan.lastjcl.scalist;

import java.nio.channels.IllegalSelectorException;
import java.util.ListIterator;
import java.util.NoSuchElementException;

import org.dan.lastjcl.scaliter.ScalIterator;

/**
 * It's returned by {@link BytesList#subList(int, int)}.
 * @author Daneel S. Yaitskov (rtfm.rtfm.rtfm@gmail.com)
 * 2012 year
 *
 */
public class ScalListIterator<E> implements ListIterator<E> {

	private ScalIterator<E> iterator;
	private int indexOfLastReturned;
	
	public ScalListIterator(ScalIterator<E> iterator) {
		this.iterator = iterator;
		indexOfLastReturned = -1;
	}	
	
	/** {@inheritDoc} */
	public boolean hasNext() {		
		return iterator.hasNext();
	}

	/** {@inheritDoc} */
	public E next() {
		indexOfLastReturned = iterator.getCurrent();
		return iterator.next();
	}
	
	/** {@inheritDoc} */
	public boolean hasPrevious() {		
		return iterator.getCurrent() > 0;
	}
	
	public E previous() {		
		int c = iterator.getCurrent();
		if (c < 1)
			throw new NoSuchElementException();
		iterator.setCurrent(indexOfLastReturned = c - 1);
		return iterator.get();
	}
	
	/** {@inheritDoc} */
	public int nextIndex() {
		return iterator.getCurrent();
	}

	/** {@inheritDoc} */
	public int previousIndex() {
		return iterator.getCurrent() - 1;
	}

	/**
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public void remove() {
		throw new UnsupportedOperationException();		
	}

	/** {@inheritDoc} */
	public void set(E e) {
		if (indexOfLastReturned < 0)			
			throw new IllegalSelectorException();
		int current = iterator.getCurrent();
		iterator.setCurrent(indexOfLastReturned);
		iterator.set(e);
		iterator.setCurrent(current);
	}

	/**
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public void add(E e) {
		throw new UnsupportedOperationException();		
	}
}
