/**
 * 
 */
package org.dan.lastjcl.scaliter;

import java.util.Collection;
import java.util.List;
import java.util.ListIterator;

import org.dan.lastjcl.IterableUtils;
import org.dan.lastjcl.scalist.ScalListIterator;
import org.dan.lastjcl.scalist.SubList;

/**
 * Class group unchanging methods that are duplicated into implementations 
 * of {@link Collection} and {@link List}.
 * @author Daneel S. Yaitskov (rtfm.rtfm.rtfm@gmail.com)
 * 2012 year
 *
 */
public abstract class AbstractIterable<T> implements Iterable<T> {

	/**
	 * If o is an iterable then compare correspond elements. 
	 * Both collections can hold {@null} elements.
	 * @param o any object. 
	 */
	@Override
	public boolean equals(Object o) {
		return IterableUtils.equals(this, o);		
	}
	
	// for collection interface
	
	/**
	 * {@inheritDoc} 
	 * @param o can be {@code null}
	 */
	public boolean contains(Object o) {
		return IterableUtils.contains(this, o);
	}
	
	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public boolean add(T e) {
		throw new UnsupportedOperationException();
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public boolean remove(Object o) {
		throw new UnsupportedOperationException();
	}

	/**
	 * {@inheritDoc}
	 * @param c can contain {@code null}
	 */
	public boolean containsAll(Collection<?> c) {
		return IterableUtils.containsAll(this, c);
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public boolean addAll(Collection<? extends T> c) {
		throw new UnsupportedOperationException();
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public boolean removeAll(Collection<?> c) {
		throw new UnsupportedOperationException();
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public boolean retainAll(Collection<?> c) {
		throw new UnsupportedOperationException();
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public void clear() {
		throw new UnsupportedOperationException();		
	}
	
	public abstract int size();
	
	// for List
	
	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public boolean addAll(int index, Collection<? extends T> c) {
		throw new UnsupportedOperationException();
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public void add(int index, T element) {
		throw new UnsupportedOperationException();		
	}

	/** 
	 * Not supported.
	 * @throws UnsupportedOperationException an array has unmodifiable structure
	 */
	public T remove(int index) {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * {@inheritDoc}
	 * @return iterator with unsupported structure modification operations.
	 */
	public ListIterator<T> listIterator() {		
		return new ScalListIterator<T>((ScalIterator<T>)iterator());
	}

	
	/**
	 * {@inheritDoc} 
	 * @return iterator with unsupported structure modification operations.
	 */
	public ListIterator<T> listIterator(int index) {
		if (index < 0 || size() > index)
			throw new IndexOutOfBoundsException(index + ""); 
		ScalIterator<T> iter = (ScalIterator<T>)iterator();		
		iter.setCurrent(index);		
		return new ScalListIterator<T>(iter);
	}
	
	/** 
	 * Don't call this method of classes that don't implement {@link List}
	 * interface.
	 * {@inheritDoc} */
	@SuppressWarnings("unchecked")
	public List<T> subList(int fromIndex, int toIndex) {		
		return new SubList<T>((List<T>)this, fromIndex, toIndex - fromIndex);
	}

}
