/**
 * 
 */
package org.dan.lastjcl.scaliter;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.dan.lastjcl.IterableUtils;

/**
 * @author Daneel S. Yaitskov (rtfm.rtfm.rtfm@gmail.com) 2012 year
 * 
 */
public class CompoundIterable<T> implements Iterable<T> {

	protected List<Iterable<T>> iterables;

	public CompoundIterable(List<Iterable<T>> iterables) {
		this.iterables = iterables;
	}

	private static class NullIterator<T> implements Iterator<T> {

		/**
		 *
		 */
		public boolean hasNext() {
			return false;
		}

		/**
		 *
		 */
		public T next() {
			throw new NoSuchElementException();
		}

		/**
		 *
		 */
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}

	private static class CompoundIterator<T> implements Iterator<T> {

		protected Iterator<T> currentIterator;
		protected int currentIterable;
		protected CompoundIterable<T> iterable;

		public void getIter() {
			if (iterable.iterables.size() > currentIterable) 
				currentIterator = iterable.iterables.get(currentIterable)
						.iterator();				
			else
				currentIterator = new NullIterator<T>();
		}
		
		public CompoundIterator(CompoundIterable<T> iterable) {			
			this.iterable = iterable;
			getIter();
		}

		/**
		 *
		 */
		public boolean hasNext() {			
			int size = iterable.iterables.size(); 
			while (size > currentIterable) {
				if (currentIterator.hasNext())
					return true;
				currentIterable += 1;
				getIter();
			}		
			return false;
		}

		/**
		 *
		 */
		public T next() {
			for (;;) {
				try {
					return currentIterator.next();					
				} catch (NoSuchElementException e) {
					if (currentIterable < iterable.iterables.size()) 
						currentIterator = iterable.iterables.get(++currentIterable).iterator();
					else
						throw e;					
				}
			}
		}

		/**
		 *
		 */
		public void remove() {
			currentIterator.remove();
		}

	}

	/**
	 *
	 */
	public Iterator<T> iterator() {
		return new CompoundIterator<T>(this);
	}
	
	
	public void appendCollections(Collection<? extends Iterable<T>> collections) {
		iterables.addAll(collections);
	}
	
	/**
	 * don't copy argument
	 * @param collection
	 */
	public void addCollection(Iterable<T> collection) {
		iterables.add(collection);
	}
	
	@Override
	public boolean equals(Object o) {
		return IterableUtils.equals(this, o);		
	}
}
